###########################################################################################
# SUPPORTING INFORMATION TO 
# Moura, M.R., Arglo, A.J., Costa, H.C. (2016) Historical and contemporary correlates of snake biogeographical subregions in the Atlantic Forest hotspot. Journal of Biogeography, XXX-XXX.
#
###########################################################################################

# STEPS IN THE R SCRIPT S1
#   1. Load the dissimilarity matrix and obtain the Euclidean representation of it
#   2. Simulate all possible values for 'k' according to the maximum number of starting groups
#   3. Inspect the variation in the values of optimal 'k' and identify the most frequent 'k'
#   4. Perform NMDS with community data and export the ordination-scores to the geographic space
#   5. Create the geographic space to receive the interpolated values of NMDS-scores
#   6. Interpolate the ordination-scores via Inverse-Distance Weighting (IDW)
#   7. Perform K-means classificaiton using the interpolated faunal dissimilarity
#   8. Get the average RGB composition for discrete visualization of biogeographic subregion
#   9. Alternatives to export layers to use in different SIG softwares

library(gstat)
library(raster)
library(rgdal)
library(sp)
library(vegan)
library(plyr)
library(doParallel)
library(foreach)
library(pgirmess)

##########################################################################################
# STEP 1 - Load the dissimilarity matrix and obtain the Euclidean representation of it

# Read dissimilarity_matrix provided as Appendix S2
dissimilarity_matrix<-read.table("Moura_et_al_Supporting_Information_Dissimilarity_matrix.txt", h=T)

# Select only the distance measures (excluding coordinates and sites number)
coords<-dissimilarity_matrix[,1:2] # for posterior use
sites_number<-dissimilarity_matrix[,3] # for posterior use
dissimilarity_matrix<-(dissimilarity_matrix[,4:ncol(dissimilarity_matrix)])
dissimilarity_matrix<-as.matrix(dissimilarity_matrix)

# Perform an ordination via Principal Coordinate Analysis (PCoA) with correction for negative eigenvalues [add = T]
ordination_output<-cmdscale(dissimilarity_matrix, k=(nrow(dissimilarity_matrix)-1), eig = F, add = T, x.ret = FALSE)
pcoa_scores<-as.data.frame(ordination_output[[1]]) # Extract the PCoA scores for each site

# Check the fit (R-squared) between observed dissimilarity and ordination distance (Shepard Diagram)
ordination_distance<-dist(pcoa_scores)
observed_dissimilarity<-as.dist(dissimilarity_matrix)
plot(ordination_distance~observed_dissimilarity, pch=1, col="gray", 
     xlab="Observed Dissimilarity", ylab="Ordination Distance (PCoA)",
     cex.lab=1.2, las=1)
summary(lm(ordination_distance~observed_dissimilarity))[8] # Get the R-squared. It should be one if PCoA uses correction for negative eigenvalues

##########################################################################################
# STEP 2 Simulate all possible values for 'k' according to the maximum number of starting groups

a<-Sys.time()
# Get values of the evaluation criterion (Within-Groups SS) for k varying from 2 to 'number of sites minus 1'
sum_of_squares <- foreach(i = 2:(ncol(dissimilarity_matrix)-1),
                          .combine = 'cbind')  %dopar% { # cbind the results of foreach
                            classification<-kmeans(pcoa_scores, i, iter.max=1000, nstart=100)
                            classification$tot.withinss} # the Within-Groups SS for k=i
y<-t(as.matrix(sum_of_squares)) # convert Within-Group SS to a vector (y for piecewise regressions)
x<-c(2:(ncol(dissimilarity_matrix)-1)) # number of clusters (x for piecewise regressions)

# Create an empty matrix to receive the values of 'max-k entered a priori' and the respective 'optimal k selected'
initial_kmax<-c(4:ncol(dissimilarity_matrix)) # At least 4 points to produce two lines (L-method)
cluster_results<-cbind(initial_kmax, c(NA)) # Dataframe to receive values of the breakpoints

# Perform multiple piecewise regression varying the max-k entered a priori from 4 to 'number of sites'
for (j in 4:ncol(dissimilarity_matrix)){ # the maximum number of 'k' established a priori
  y_selected<-y[1:j] # Limit the number of starting points (y axis)
  x_selected<-x[1:j] # Limit the number of starting points (x axis)
  find_the_knee <- foreach(i = 2:(j-1), # calculate the RSE for all possible breakpoints (i) when starting with k-max = j
                           .combine = 'cbind')  %dopar% { # 
                             piecewise_k<-lm(y_selected~x_selected*(x_selected<i) + x_selected*(x_selected>=i))
                             summary(piecewise_k)[6]} # print the RSE for each piecewise regression
  find_the_knee<-c(do.call("cbind",find_the_knee)) # convert the previous output (list) to vector
  knee_found<-as.data.frame(cbind(c(2:(j-1)), find_the_knee)) # combine position of breakpoint and RSE value
  names(knee_found)<-c("breakpoint", "RSE") # rename
  knee_found<-na.omit(knee_found) # remove NaN values
  k_number<-knee_found[which(knee_found$RSE==min(knee_found$RSE)),1] # select breakpoint that minimizes RSE 
  cluster_results[(j-3),2]<-k_number} # print the maximum number of initial groups (j) and the respective selected breakpoint (k)
(Sys.time()-a) # How long it takes?

##########################################################################################
# STEP 3 Inspect the variation in the values of optimal 'k' and identify the most frequent 'k'

# Plot the optimal 'k' against the number of initial groups tested (maximum number of groups)
plot(cluster_results[,2]~cluster_results[,1], las=1, xlab="Number of initial groups tested",
     ylab="Optimal k selected", cex.lab=1.2, bty="l", col="gray90")
lines(cluster_results[,1], cluster_results[,2], type = "l")
segments(0,6,200,6, col="red", lty=2)

# Identify the most frequent 'k' (math-mode)
most_frequent_k<-as.data.frame(cluster_results)[,2]
optimal_k<-names(sort(-table(most_frequent_k)))[1]
optimal_k # what is the most frequent value?
hist(most_frequent_k, n=50, ylab="Frequency of k", xlab="Value of the optimal k", main="", cex.axis=1.0, cex.lab=1.2, las=1, col="grey")

##########################################################################################
# STEP 4 - Perform NMDS with community data and extract the ordination-scores for each site

# Ordination via Non-Metric Multidimensional Scaling (NMDS)
set.seed(123)
ordination_output<-metaMDS(dissimilarity_matrix, center=T, k=3, trace=F, trimax=10000)

# Check the linear and non-lienar fit between observed dissimilarity and ordination distance (Shepard Diagram)
stressplot(ordination_output, pch=1, p.col = "gray", l.col = "black", lwd=2, las=1, cex.lab=1.2)

# Extract NMDS-scores and bind them to the geographic coordinates of sites
nmds_scores<-scores(ordination_output, display = "sites")

# Plot the Moran's I correlogram for the NMDS axes
pgi.cor<-as.data.frame(correlog(coords=coords, z=nmds_scores[,1], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(18, 1, c("NMDS - Axis 1"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=nmds_scores[,2], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(18, 1, c("NMDS - Axis 2"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=nmds_scores[,3], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(18, 1, c("NMDS - Axis 3"), pch=16, lty=1, bty="n", cex=0.9)

# Convert NMDS-scores to SpatialPoints objects
nmds_scores<-cbind(coords, nmds_scores)
coordinates(nmds_scores)<-~longitude+latitude # Convert 'nmds_scores' dataframe to SpatialPointsDataframe
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(nmds_scores)<-wgs84 # Set the original CRS (WGS84)
new_crs<-"+proj=robin +lon_0=0 +x_0=0 +y_0=0 +a=6371000 +b=6371000 +units=m +no_defs" # Transform CRS to Robinson projection (it is necessary to have projected data for interpolation)
nmds_scores<-spTransform(nmds_scores, new_crs)
  
##########################################################################################
# STEP 5 - Create the geographic space to receive the interpolated values of NMDS-scores

# Download  the shapefile of Brazilian Atlantic Forest (BAF) at <http://www.mma.gov.br/estruturas/202/_arquivos/shape_mata_atlantica_ibge_5milhoes_policonica_sirgas2000shp_202.zip>. Please, unzip it in a folder called 'shapes'

# Read the shapefile of Brazilian Atlantic Forest (BAF) and rasterize it
BAF_limit<-readOGR(dsn='shapes',layer='shape_mata_atlantica_IBGE_5milhoes_policonica_sirgas2000')
BAF_limit<-spTransform(BAF_limit, wgs84) # Change the CRS from SIRGAS2000 to WGS84
r_ext<-extent(BAF_limit)
r<-raster() # Create a empty raster to receive data
res(r)<-0.16667 # Define raster resolution in decimal degrees (= 10 arc-min)
r<-crop(r, r_ext) # Define spatial extent of the raster (same as BAF)
BAF_raster<-rasterize(BAF_limit, r, field=1, background=NA)

# Get the spatial coordinates of BAF pixels
get_coords<-rasterToPoints(BAF_raster)
get_coords<-as.data.frame(get_coords[, c(1:2)])
get_coords<-cbind(get_coords, NA, NA, NA)
names(get_coords)<-c("longitude", "latitude", "NMDS1", "NMDS2", "NMDS3")
coordinates(get_coords)<-~longitude+latitude # Convert 'get_coords' dataframe to SpatialPointsDataframe
crs(get_coords)<-wgs84 # Set the original CRS (WGS84)
get_coords<-spTransform(get_coords, new_crs) # Transform CRS to Robinson projection

##########################################################################################
# STEP 6 - Interpolate the ordination-scores via Inverse-Distance Weighting (IDW)

# Interpolate first ordination axis using Inverse-Distance Weighting and convert to raster
nmds_interpol<-idw(NMDS1~1, locations=nmds_scores, newdata=get_coords, idp=2) #idp = power of inverse distance function
nmds_interpol<-spTransform(nmds_interpol, wgs84) # Return CRS to WGS84 after the interpolation
NMDS1_idw_raster <- rasterize(nmds_interpol, r, field=nmds_interpol@data$var1.pred, background=NA)

# Same as above but for second and third ordination axes
nmds_interpol<-idw(NMDS2~1, locations=nmds_scores, newdata=get_coords, idp=2) 
nmds_interpol<-spTransform(nmds_interpol, wgs84)
NMDS2_idw_raster <- rasterize(nmds_interpol, r, field=nmds_interpol@data$var1.pred, background=NA)

nmds_interpol<-idw(NMDS3~1, locations=nmds_scores, newdata=get_coords, idp=2) 
nmds_interpol<-spTransform(nmds_interpol, wgs84)
NMDS3_idw_raster <- rasterize(nmds_interpol, r, field=nmds_interpol@data$var1.pred, background=NA)

# Create a stack raster with the three interpolated layers
IDW_stack<-stack(NMDS1_idw_raster, NMDS2_idw_raster, NMDS3_idw_raster)

# Extract IDW estimations at each observed site
observed_sites<-SpatialPoints(cbind(coords$longitude, coords$latitude))
IDW_estimated_values <- foreach(i = iter(names(IDW_stack)), .combine='cbind',
                                .packages = c("raster")) %dopar% {
                                  extract(IDW_stack[[i]], # raster object
                                          observed_sites, # vector coordinates, spatial polygons/points/lines
                                          buffer=10000, # buffer of 10 min-arc
                                          fun=mean, # summarize the raster values 
                                          na.rm=TRUE, # remove NA values before 'fun'
                                          df=T)[2]}

# Comparison among observed and estimate values
comparison<-cbind(nmds_scores@data, IDW_estimated_values)
names(comparison)<-c("Obs1", "Obs2", "Obs3", "Est1", "Est2", "Est3")
summary(lm(Est1~Obs1, data=comparison))[8] # R-squared between observed vs. estimated NMDS-scores - Axis1
summary(lm(Est2~Obs2, data=comparison))[8] # R-squared between observed vs. estimated NMDS-scores - Axis2
summary(lm(Est3~Obs3, data=comparison))[8] # R-squared between observed vs. estimated NMDS-scores - Axis3

# Plot the NMDS axes interpolated via Inverse-Distance Weighting (IDW) and Ordinary Kriging (OK) and variance erro for OK
plot(NMDS1_idw_raster, main="IDW - 1st axis")
plot(NMDS2_idw_raster, main="IDW - 2nd axis")
plot(NMDS3_idw_raster, main="IDW - 3rd axis")

##########################################################################################
# STEP 7. Perform K-means classificaiton using the interpolated faunal dissimilarity

# Create a RGB raster using each interpolated-axis as a raster band
IDW_stack<-stack(NMDS1_idw_raster, NMDS2_idw_raster, NMDS3_idw_raster)

# Extract the raster values from each raster-stack layer in parallel and write these into a dataframe
cl <- makePSOCKcluster(detectCores()-1)
registerDoParallel(cl)
getDoParWorkers() 
iters <- length(names(IDW_stack))
RGB_IDW_matrix <- foreach(i = 1:iters,
                          .combine = 'cbind',
                          .packages = c("raster"))  %dopar% { # definy what packages would be used for computations
                            raster_to_points<- rasterToPoints(unlist(IDW_stack)[[i]])
                            raster_to_points[,3]}

# Get the spatial coordinates of each pixel
get_coords<-rasterToPoints(unlist(IDW_stack)[[1]])
get_coords<-get_coords[, c(1:2)]

# Perform K-means clustering analysis using the optimal number of 'k'
set.seed(123)
classification<-kmeans(RGB_IDW_matrix, 6, iter.max = 50000, nstart = 5000) # K = 6
Subregions_IDW<-cbind(get_coords, classification$cluster)

##########################################################################################
# STEP 8. Get the average RGB composition for discrete visualization of biogeographic subregion

# Normalize the interpolated axes between 0 and 1 using (x - min(x))/(max(x) - min(x))
colour_IDW<-(apply(RGB_IDW_matrix, MARGIN = 2, FUN = function(X) (X - min(X))/diff(range(X))))*255

# Combine RGB colour with cluster id for each grid cell 
colour_IDW<-as.data.frame(cbind(Subregions_IDW, colour_IDW))
names(colour_IDW)<-c("longitude", "latitude", "subregion", "R", "G", "B")
colour_IDW$subregion<-as.factor(colour_IDW$subregion) # Put the subregion-number as factor

# Obtain the median RGB composition for each biogeographic subregion
mean_R<-ddply(colour_IDW,~subregion, summarise, mean=mean(R))
mean_G<-ddply(colour_IDW,~subregion, summarise, mean=mean(G))
mean_B<-ddply(colour_IDW,~subregion, summarise, mean=mean(B))
mean_colour_IDW<-cbind(mean_R, mean_G[,2], mean_B[,2])
names(mean_colour_IDW)<-c("subregion", "R", "G", "B")

# Assign the mean RGB colour for each pixel
subregion_col<-as.data.frame(Subregions_IDW)
R<-rep(NA, nrow(subregion_col))
G<-rep(NA, nrow(subregion_col))
B<-rep(NA, nrow(subregion_col))
subregion_col<-cbind(subregion_col, R, G, B)
for (i in 1:nrow(mean_colour_IDW)){
  select_rows <- which(subregion_col$V3==i)   # Column V3 define the subregion number    
  subregion_col[select_rows, 4] <- mean_colour_IDW[i, 2] # set the value of R
  subregion_col[select_rows, 5] <- mean_colour_IDW[i, 3] # set the value of G
  subregion_col[select_rows, 6] <- mean_colour_IDW[i, 4]} # set the value of B

# Rasterize the RGB layers and stack them in a RGB raster
IDW_subregion_R <- rasterFromXYZ(subregion_col[,c(1,2,4)], res=c(NA, NA), digits=6)
IDW_subregion_G <- rasterFromXYZ(subregion_col[,c(1,2,5)], res=c(NA, NA), digits=6)
IDW_subregion_B <- rasterFromXYZ(subregion_col[,c(1,2,6)], res=c(NA, NA), digits=6)
IDW_discrete_col<-stack(IDW_subregion_R, IDW_subregion_G, IDW_subregion_B)
crs(IDW_discrete_col)<-wgs84

# Plot the tiogeographic regionalization of interpolated dissimilarities
op <- par(mfrow = c(1, 2))
plotRGB(IDW_discrete_col, r=1, g=2, b=3, stretch='hist') # 'Discrete' RGB-colours for IDW interpolation 
plotRGB(IDW_stack, r=1, g=2, b=3, stretch='hist') # 'Continuous' RGB-colours for IDW interpolation
op <- par(mfrow = c(1, 1))

##########################################################################################
# STEP 9. Alternatives to export layers to use in different SIG softwares

# Save the interpolated rasters
writeRaster(IDW_stack, "Moura_et_al_IDW_RGB.img", format="HFA", overwrite=T) # 'Continuous' RGB-colours from IDW interpolation
writeRaster(IDW_discrete_col, "IDW_discrete_RGB.img", format="HFA", overwrite=T) # 'Discrete' RGB-colours from OK interpolation

# Save a single raster layer with discrete values representing each biogeographic subregions 
Subregions_IDW <- rasterFromXYZ(Subregions_IDW, res=c(NA, NA), digits=6)
crs(Subregions_IDW)<-wgs84 # Assign coordinate reference system
writeRaster(Subregions_IDW, "Moura_et_al_IDW_Subregions.img", format="HFA", overwrite=T)

# Save biogeographic subregions as shapefiles
IDW_shapefile<-rasterToPolygons(Subregions_IDW, fun=NULL, n=4, na.rm=TRUE, digits=12, dissolve=T)
writeOGR(IDW_shapefile, ".", "Moura_et_al_IDW_regionalization", driver="ESRI Shapefile")