###########################################################################################
# SUPPORTING INFORMATION TO 
# Moura, M.R., Arglo, A.J., Costa, H.C. (2016) Historical and contemporary correlates of snake biogeographical subregions in the Atlantic Forest hotspot. Journal of Biogeography, XXX-XXX.
#
###########################################################################################

# STEPS IN THE R SCRIPT S2
#   1. Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM CCSM4
#   2. Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM MIROC-ESM
#   3. Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM MPI-ESM-P
#   4. Get the average of the layers of historical stability in thermal and hydric conditions built in step 1 to 3
#   5. Get the explanatory variables at each site
#   6. Evaluate multicollinearity among explanatory variables

# Load needed packages
library(rgdal)
library(raster)
library(rgeos)
library(doParallel)
library(vegan)
library(pgirmess)
library(colorRamps)
library(usdm)

removeTmpFiles(0.01) # will remove tmp files)
rasterOptions(overwrite = T, progress = "text",timer = T, tolerance = 0.5, maxmemory = 5e+08,chunksize = 5e+09)
beginCluster(detectCores() -1) #use all but one core

##########################################################################################
# STEP 1 - Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM CCSM4

# Download  the shapefile of Brazilian Atlantic Forest (BAF) at <http://www.mma.gov.br/estruturas/202/_arquivos/shape_mata_atlantica_ibge_5milhoes_policonica_sirgas2000shp_202.zip>. Please, unzip it in a folder called 'shapes'

# Download biclimatic variables at 2.5 arc-min resolution for the current time and  the three scenarios of Last Glacial Maximum.

# Read the shapefile of BAF and rasterize it
BAF_boundaries<-readOGR(dsn='shapes',layer='shape_mata_atlantica_IBGE_5milhoes_policonica_sirgas2000')
ceqa<-"+proj=cea +lon_0=Central Meridian +lat_ts=Standard Parallel +x_0=False Easting +y_0=False Northing"
BAF_boundaries<-spTransform(BAF_boundaries, ceqa) # Change the CRS from SIRGAS2000 to Cylindrical Equal Area projection
BAF_buffered<-gBuffer(BAF_boundaries, byid=T, width=25000, quadsegs=5, capStyle="ROUND", joinStyle="ROUND") # Buffer 25km
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
BAF_buffered<-spTransform(BAF_buffered, wgs84) # Change the CRS from SIRGAS2000 to WGS84
r_ext<-extent(BAF_buffered)
writeOGR(BAF_buffered, "shapefile", "BAF_buffered", driver="ESRI Shapefile") # Save the buffered BAF for posterior use

# Read a stack raster with bioclim variables for the current time at 2.5 arc-min
path_2_bioclim_current<-"C:/SIG/Raster/Worldclim/Current_2.5min_arc" # Specify the path to bioclim layers for current time
path_2_bioclim_past<-"C:/SIG/Raster/Worldclim/LGM_21k_2.5min_arc_CCSM4" # Specify the path to bioclim layers for LGM CCSM4
bioclim_current<-stack(list.files(path = path_2_bioclim_current, pattern='bil', full.names=T))
bioclim_lgm<-stack(list.files(path = path_2_bioclim_past, pattern='tif', full.names=T))

# Adjust extent of the global raster stacks to meet extent of only BAF
bioclim_current<-crop(bioclim_current, r_ext)
bioclim_current<-mask(bioclim_current, BAF_buffered) # disconsider raster cells outside BAF
bioclim_lgm<-crop(bioclim_lgm, r_ext)
bioclim_lgm<-mask(bioclim_lgm, BAF_buffered) # disconsider raster cells outside BAF

# Get raster layers of the difference between bioclimatic variables for current and LGM conditions (in parallel) 
cl<-makePSOCKcluster(detectCores()-1) # leave one core free (so, your computer will not run too slow)
registerDoParallel(cl) # register parallel backend
delta_layers<-foreach(i = 1:length(names(bioclim_current)),
                      .packages = c("raster"))  %dopar% { # definy what package should be used for computations
                      to.raster<- bioclim_current[[i]] - bioclim_lgm[[i]]}

# Extract the values of 'delta-bioclimatic' variables to a matrix
delta_matrix<-foreach(i = 1:length(names(bioclim_current)),
                        .combine = 'cbind',
                        .packages = c("raster"))  %dopar% {
                          raster_to_points<- rasterToPoints(unlist(delta_layers)[[i]])
                          raster_to_points[,3]}
stopCluster(cl)

# Perform a Principal Component Analysis (PCA) using 'delta-bioclimatic' variables
delta_temp<-delta_matrix[, c(1,2,3,12,13,14,15,16,17,18,19)] # to filter just temperature related variables
delta_prec<-delta_matrix[, 4:11] # to filter just precipitation related variables
pca_scores_temp<-(prcomp(delta_temp, center=T, scale=T))[[5]] # PCA scores for 'delta-bioclimatic' temperature-related variables
pca_scores_prec<-(prcomp(delta_prec, center=T, scale=T))[[5]] # Same as above, but for precipitation-related variables

# Get the Euclidean distance between the each PCA-score and the PCA-axes origin in the multidimensional ordination space
sq_dist_temp<-(pca_scores_temp)^2 # squared the PCA scores of each axis (calculation of Euclidean distance)
sq_dist_prec<-(pca_scores_prec)^2
eucl_dist_temp<-matrix(nrow=(nrow(sq_dist_temp)), ncol=1) # empty matrix to 'for loop' outputs in the following
eucl_dist_prec<-matrix(nrow=(nrow(sq_dist_prec)), ncol=1)
for (i in 1:nrow(eucl_dist_temp)){ # calculate the Euclidean distance between PCA scores and axes-origin
  squared_dist_summed<-sum(sq_dist_temp[i,])
  eucl_dist_temp[i,1]<-sqrt(squared_dist_summed)}
for (i in 1:nrow(eucl_dist_prec)){
  squared_dist_summed<-sum(sq_dist_prec[i,])
  eucl_dist_prec[i,1]<-sqrt(squared_dist_summed)}

# Rasterize the historical climatic stability based on LGM CCSM4
coords<-(rasterToPoints(unlist(delta_layers)[[1]]))[, 1:2] # Geographic coordinates of the raster grid cells
eucl_dist_temp<-cbind(coords,eucl_dist_temp) # Prepare data to rasterisation 
eucl_dist_prec<-cbind(coords,eucl_dist_prec)
HTC<-rasterFromXYZ(eucl_dist_temp, res=c(NA, NA), digits=6)
HHC<-rasterFromXYZ(eucl_dist_prec, res=c(NA, NA), digits=6)
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(HTC)<-wgs84 # Assign coordinate reference system
crs(HHC)<-wgs84
dir.create("raster", showWarnings = F)

writeRaster(delta_layers[[1]], "raster/deltaAMT_CCSM4.img", format="HFA", overwrite=T) # Save as raster file
writeRaster(delta_layers[[4]], "raster/deltaAPP_CCSM4.img", format="HFA", overwrite=T)
writeRaster(HTC, "raster/HTC_CCSM4.img", format="HFA", overwrite=T) # Save as raster file
writeRaster(HHC, "raster/HHC_CCSM4.img", format="HFA", overwrite=T)

##########################################################################################################################
# STEP 2 - Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM MIROC-ESM
rm(list=ls())

# Every comand line below is identical to those above, with exception of the GCM used for past conditions
BAF_buffered <- readOGR(dsn='shapefile',layer='BAF_buffered')
r_ext<-extent(BAF_buffered)
path_2_bioclim_current<-"C:/SIG/Raster/Worldclim/Current_2.5min_arc" # Remember to change the path as done in Step 1
bioclim_current<-stack(list.files(path = path_2_bioclim_current, pattern='bil', full.names=T))
path_2_bioclim_past<-"C:/SIG/Raster/Worldclim/LGM_21k_2.5min_arc_MIROC-ESM" # THIS IS THE ONLY DIFFERENCE. Change the path
bioclim_lgm <- stack(list.files(path = path_2_bioclim_past, pattern='tif', full.names=T))
bioclim_current <- crop(bioclim_current, r_ext)
bioclim_current <- mask(bioclim_current, BAF_buffered)
bioclim_lgm <- crop(bioclim_lgm, r_ext)
bioclim_lgm <- mask(bioclim_lgm, BAF_buffered)
cl <- makePSOCKcluster(detectCores()-1)
registerDoParallel(cl)
delta_layers <- foreach(i = 1:length(names(bioclim_current)),
                        .packages = c("raster"))  %dopar% {
                          to.raster<- bioclim_current[[i]] - bioclim_lgm[[i]]}
delta_matrix <- foreach(i = 1:length(names(bioclim_current)),
                        .combine = 'cbind',
                        .packages = c("raster"))  %dopar% {
                          raster_to_points<- rasterToPoints(unlist(delta_layers)[[i]])
                          raster_to_points[,3]}
stopCluster(cl)
delta_temp<-delta_matrix[, c(1,2,3,12,13,14,15,16,17,18,19)] # to filter just temperature related variables
delta_prec<-delta_matrix[, 4:11] # to filter just precipitation related variables
pca_scores_temp<-(prcomp(delta_temp, center=T, scale=T))[[5]]
pca_scores_prec<-(prcomp(delta_prec, center=T, scale=T))[[5]]
sq_dist_temp<-(pca_scores_temp)^2
sq_dist_prec<-(pca_scores_prec)^2
eucl_dist_temp<-matrix(nrow=(nrow(sq_dist_temp)), ncol=1)
eucl_dist_prec<-matrix(nrow=(nrow(sq_dist_prec)), ncol=1)
for (i in 1:nrow(eucl_dist_temp)){
  squared_dist_summed<-sum(sq_dist_temp[i,])
  eucl_dist_temp[i,1]<-sqrt(squared_dist_summed)}
for (i in 1:nrow(eucl_dist_prec)){
  squared_dist_summed<-sum(sq_dist_prec[i,])
  eucl_dist_prec[i,1]<-sqrt(squared_dist_summed)}
coords<-(rasterToPoints(unlist(delta_layers)[[1]]))[, 1:2]
eucl_dist_temp<-cbind(coords,eucl_dist_temp) 
eucl_dist_prec<-cbind(coords,eucl_dist_prec)
HTC<-rasterFromXYZ(eucl_dist_temp, res=c(NA, NA), digits=6)
HHC<-rasterFromXYZ(eucl_dist_prec, res=c(NA, NA), digits=6)
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(HTC)<-wgs84
crs(HHC)<-wgs84
writeRaster(delta_layers[[1]], "raster/deltaAMT_MIROC_ESM.img", format="HFA", overwrite=T) # Save as raster file
writeRaster(delta_layers[[4]], "raster/deltaAPP_MIROC_ESM.img", format="HFA", overwrite=T)
writeRaster(HTC, "raster/HTC_MIROC_ESM.img", format="HFA", overwrite=T)
writeRaster(HHC, "raster/HHC_MIROC_ESM.img", format="HFA", overwrite=T)

##########################################################################################################################
# STEP 3 - Create raster layers of historical stability in thermal and hydric conditions using the general circulation model LGM MPI-ESM-P
rm(list=ls())

# Every comand line below is identical to those above, with exception of the GCM used for past conditions
BAF_buffered <- readOGR(dsn='shapefile',layer='BAF_buffered')
r_ext<-extent(BAF_buffered)
path_2_bioclim_current<-"C:/SIG/Raster/Worldclim/Current_2.5min_arc" # Remember to change the path as done in Step 1
bioclim_current <- stack(list.files(path = path_2_bioclim_current, pattern='bil', full.names=T))
path_2_bioclim_past <- "C:/SIG/Raster/Worldclim/LGM_21k_2.5min_arc_MPI-ESM-P" # THIS IS THE ONLY DIFFERENCE. Change the path
bioclim_lgm <- stack(list.files(path = path_2_bioclim_past, pattern='tif', full.names=T))
bioclim_current <- crop(bioclim_current, r_ext)
bioclim_current <- mask(bioclim_current, BAF_buffered)
bioclim_lgm <- crop(bioclim_lgm, r_ext)
bioclim_lgm <- mask(bioclim_lgm, BAF_buffered)
cl <- makePSOCKcluster(detectCores()-1)
registerDoParallel(cl)
delta_layers <- foreach(i = 1:length(names(bioclim_current)),
                        .packages = c("raster"))  %dopar% {
                          to.raster<- bioclim_current[[i]] - bioclim_lgm[[i]]}
delta_matrix <- foreach(i = 1:length(names(bioclim_current)),
                        .combine = 'cbind',
                        .packages = c("raster"))  %dopar% {
                          raster_to_points<- rasterToPoints(unlist(delta_layers)[[i]])
                          raster_to_points[,3]}
stopCluster(cl)
delta_temp<-delta_matrix[, c(1,2,3,12,13,14,15,16,17,18,19)] # to filter just temperature related variables
delta_prec<-delta_matrix[, 4:11] # to filter just precipitation related variables
pca_scores_temp<-(prcomp(delta_temp, center=T, scale=T))[[5]]
pca_scores_prec<-(prcomp(delta_prec, center=T, scale=T))[[5]]
sq_dist_temp<-(pca_scores_temp)^2
sq_dist_prec<-(pca_scores_prec)^2
eucl_dist_temp<-matrix(nrow=(nrow(sq_dist_temp)), ncol=1)
eucl_dist_prec<-matrix(nrow=(nrow(sq_dist_prec)), ncol=1)
for (i in 1:nrow(eucl_dist_temp)){
  squared_dist_summed<-sum(sq_dist_temp[i,])
  eucl_dist_temp[i,1]<-sqrt(squared_dist_summed)}
for (i in 1:nrow(eucl_dist_prec)){
  squared_dist_summed<-sum(sq_dist_prec[i,])
  eucl_dist_prec[i,1]<-sqrt(squared_dist_summed)}
coords<-(rasterToPoints(unlist(delta_layers)[[1]]))[, 1:2]
eucl_dist_temp<-cbind(coords,eucl_dist_temp) 
eucl_dist_prec<-cbind(coords,eucl_dist_prec)
HTC<-rasterFromXYZ(eucl_dist_temp, res=c(NA, NA), digits=6)
HHC<-rasterFromXYZ(eucl_dist_prec, res=c(NA, NA), digits=6)
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(HTC)<-wgs84
crs(HHC)<-wgs84
writeRaster(delta_layers[[1]], "raster/deltaAMT_MPI_ESM_P.img", format="HFA", overwrite=T) # Save as raster file
writeRaster(delta_layers[[4]], "raster/deltaAPP_MPI_ESM_P.img", format="HFA", overwrite=T)
writeRaster(HTC, "raster/HTC_MPI_ESM_P.img", format="HFA", overwrite=T)
writeRaster(HHC, "raster/HHC_MPI_ESM_P.img", format="HFA", overwrite=T)

##########################################################################################################################
# STEP 4 - Get the average of the layers of historical stability in thermal and hydric conditions built in step 1 to 3
rm(list=ls())

# For each general circulation model, load the historic climatic stability for temperature (HTC) and precipitation (HHC)
HTC_CCSM4 <- raster("raster/HTC_CCSM4.img", package="raster")
HTC_MIROC_ESM <- raster("raster/HTC_MIROC_ESM.img", package="raster")
HTC_MPI_ESM_P <- raster("raster/HTC_MPI_ESM_P.img", package="raster")
HHC_CCSM4 <- raster("raster/HHC_CCSM4.img", package="raster")
HHC_MIROC_ESM <- raster("raster/HHC_MIROC_ESM.img", package="raster")
HHC_MPI_ESM_P <- raster("raster/HHC_MPI_ESM_P.img", package="raster")

# Get the average value for each raster grid cell
HTC_mean<-(sum(HTC_CCSM4, HTC_MIROC_ESM, HTC_MPI_ESM_P))/3
HHC_mean<-(sum(HHC_CCSM4, HHC_MIROC_ESM, HHC_MPI_ESM_P))/3
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(HTC_mean)<-wgs84
crs(HHC_mean)<-wgs84
writeRaster(HTC_mean, "raster/HTC_mean.img", format="HFA", overwrite=T)
writeRaster(HHC_mean, "raster/HHC_mean.img", format="HFA", overwrite=T)

# Visualization of historical stablity of temperature (HTC) and precipitation (HHC) within BAF
op <- par(mfrow = c(1, 2))
plot(HTC_mean, col=matlab.like(20)) # colorRampPalette(c("forestgreen", "yellow", "red"))(255)
plot(HHC_mean, col=matlab.like(20))
op <- par(mfrow = c(1, 1))

# For each general circulation model, load the delta between temperature and precipitation from current and LGM conditions
HDT_CCSM4 <- raster("raster/deltaAMT_CCSM4.img", package="raster")
HDT_MIROC_ESM <- raster("raster/deltaAMT_MIROC_ESM.img", package="raster")
HDT_MPI_ESM_P <- raster("raster/deltaAMT_MPI_ESM_P.img", package="raster")
HDP_CCSM4 <- raster("raster/deltaAPP_CCSM4.img", package="raster")
HDP_MIROC_ESM <- raster("raster/deltaAPP_MIROC_ESM.img", package="raster")
HDP_MPI_ESM_P <- raster("raster/deltaAPP_MPI_ESM_P.img", package="raster")

# Get the average value for each raster grid cell
HDT_mean<-(sum(HDT_CCSM4, HDT_MIROC_ESM, HDT_MPI_ESM_P))/3
HDP_mean<-(sum(HDP_CCSM4, HDP_MIROC_ESM, HDP_MPI_ESM_P))/3
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0" # WGS84 Coordinate Reference System (CRS)
crs(HDT_mean)<-wgs84
crs(HDP_mean)<-wgs84
writeRaster(HDT_mean, "raster/HDT_mean.img", format="HFA", overwrite=T)
writeRaster(HDP_mean, "raster/HDP_mean.img", format="HFA", overwrite=T)

# Visualization of historical stablity of temperature (HTC) and precipitation (HHC) within BAF
op <- par(mfrow = c(1, 2))
plot(HDT_mean, col=matlab.like(20)) # colorRampPalette(c("forestgreen", "yellow", "red"))(255)
plot(HDP_mean, col=matlab.like(20))
op <- par(mfrow = c(1, 1))

##########################################################################################################################
# STEP 5 - Get the explanatory variables at each site
rm(list=ls())

# Get the geographic coordinates of each site and create a SpatialPointDataframe
coords<-read.table("Moura_et_al_Supporting_Information_Dissimilarity_matrix.txt", h=T)[,1:2]
sites_coordinates<-SpatialPointsDataFrame(coords=coords, data=coords)
wgs84<-"+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
crs(sites_coordinates)<-wgs84 # Assign coordinate reference system

# Climate 'per se' variables
raster1<-raster("C:/SIG/Raster/Worldclim/Current_30arc-sec/bio_1.bil", package="raster") # Change the directory as needed
AMT<-extract(raster1, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Annual Mean Temperature
raster1<-raster("C:/SIG/Raster/Worldclim/Current_30arc-sec/bio_12.bil", package="raster")
APP<-extract(raster1, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Annual Precipitation

# Seasonality-related variables
raster1<-raster("C:/SIG/Raster/Worldclim/Current_30arc-sec/bio_7.bil", package="raster")
TAR<-extract(raster1, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Temperatue Annual Range
Bio16<-raster("C:/SIG/Raster/Worldclim/Current_30arc-sec/bio_16.bil", package="raster")
Bio17<-raster("C:/SIG/Raster/Worldclim/Current_30arc-sec/bio_17.bil", package="raster")
PPR<-(extract(Bio16, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2])-(extract(Bio17, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2]) # Precipitation Range

# Topographic-related variables
Elev<-raster("C:/SIG/Raster/Worldclim/alt_30arcsec.bil", package="raster")
ElevM<-extract(Elev, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Mean elevation
ElevR<-(extract(Elev, sites_coordinates, buffer=10000, fun=max, na.rm=TRUE,df=T)[2])-(extract(Elev, sites_coordinates, buffer=2500, fun=min, na.rm=TRUE,df=T)[2]) # Elevational range
ElevCV<-(extract(Elev, sites_coordinates, buffer=10000, fun=sd, na.rm=TRUE,df=T)[2])/ElevM # Roughness

# Historic climatic stability variables 
HTC_raster<-raster("raster/HTC_mean.img", package="raster")
HTC<-extract(HTC_raster, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Historical temperature stability
HHC_raster<-raster("raster/HHC_mean.img", package="raster")
HHC<-extract(HHC_raster, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Historical precipitation stability
HDT_raster<-raster("raster/HDT_mean.img", package="raster")
HDT<-extract(HDT_raster, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Historical temperature stability
HDP_raster<-raster("raster/HDP_mean.img", package="raster")
HDP<-extract(HDP_raster, sites_coordinates, buffer=10000, fun=mean, na.rm=TRUE,df=T)[2] # Historical temperature stability

# Combine all explanatory variables and rename as in the main text
variables<-cbind(coords, AMT, APP, TAR, PPR, ElevR, ElevCV, HTC, HHC, HDT, HDP)
names(variables)<-c("longitude", "latitude", "AMT", "APP", "TAR", "PPR", "ElevR", "ElevCV", "HTC", "HHC", "HDT", "HDP")

##########################################################################################################################
# STEP 6 - Evaluate multicollinearity among explanatory variables

rm(list=setdiff(ls(), "variables"))
variables<-variables[,c(3:ncol(variables))] # Remove geografic coordinates

vif(variables)
pearson_r<-as.data.frame(cor(variables, use="complete.obs")) # Pearson correlation among orthogonal polynomials
range(as.dist(pearson_r)) # Highest values for negative and positive pairwise correlations

# Save explanatory variables
write.table(variables, "Moura_et_al_Supporting_Information_Predictors.txt", sep="\t", col.names=T)
