###########################################################################################
# SUPPORTING INFORMATION TO 
# Moura, M.R., Arglo, A.J., Costa, H.C. (2016) Historical and contemporary correlates of snake biogeographical subregions in the Atlantic Forest hotspot. Journal of Biogeography, XXX-XXX.
#
###########################################################################################

# STEPS IN THE R SCRIPT S3
#   1. Compute the Multinomial Logistic Regression (MLR) using single predictor models
#   2. Compute the Multinomial Logistic Regression (MLR) using multiple predictors models
#   3. Deviance partitioning among climatic stability, productivity, and topographic complexity
#   4. Deviance partitioning among current climate, historical variation in climate, and topography
#   5. Examine the spatial structure in the residuals of the best supported multinomial logistic regression model

# Load needed packages
library(MuMIn)
library(nnet)
library(usdm)
library(pgirmess)

##########################################################################################################################
# STEP 1 - Compute the Multinomial Logistic Regression (MLR) using single predictor models

# Read tables provided as Supporting Information
rawdata<-read.table("Moura_et_al_Supporting_Information_Data_analysis.txt", h=T)
subregion<-rawdata[,3] # Extract vector of biogeographic subregions
variables<-rawdata[,-c(1:3)] # Extract dataframe with predictors

# Verify the total deviance (null model)
Null_model<-multinom(subregion~1, data=variables)
Total_deviance<-Null_model[[25]] # store the total deviance

# Calculate the percentage of deviance explained by each variable
MLRresults<-as.data.frame(matrix(nrow=10, ncol=4))
colnames(MLRresults) <- (c("Variable", "Deviance", "AIC", "AICc"))
for (i in 1:10){
  MLR_test<-multinom(subregion~variables[,i], data=variables)
  MLRresults[i,1]<-colnames(variables)[i] # store the variable name
  MLRresults[i,2]<-(Total_deviance-MLR_test[[19]])/Total_deviance # store the % of deviance explained
  MLRresults[i,3]<-MLR_test[[26]] # store the AIC value
  MLRresults[i,4]<-AICc(MLR_test)} # store the AICc value

wAICc<-Weights(MLRresults$AICc) # AIC weigth
MLRresults<-cbind(MLRresults, wAICc) # combine results in a single table
MLRresults<-MLRresults[order(MLRresults$wAICc, decreasing=T), ] # Order variables according to individual model fit 
View(MLRresults)

##########################################################################################################################
# STEP 2 - Compute the Multinomial Logistic Regression (MLR) using multiple predictors models

# Prepare the models to be tested
formlist <- list() # Empty list to receive nested formulas in forward selection
formlist[[1]]<-as.formula("subregion~AMT+APP") # Productivity
formlist[[2]]<-as.formula("subregion~PPR+TAR") # Climatic stability
formlist[[3]]<-as.formula("subregion~ElevR+ElevCV") # Topography
formlist[[4]]<-as.formula("subregion~AMT+APP+TAR+PPR") # Productivity + Climatic stability
formlist[[5]]<-as.formula("subregion~AMT+APP+ElevR+ElevCV") # Productivity + Topography
formlist[[6]]<-as.formula("subregion~TAR+PPR+ElevR+ElevCV") # Climatic stability + Topography
formlist[[7]]<-as.formula("subregion~AMT+TAR+PPR+APP+ElevR+ElevCV") # Total Contemporary Environment

# Prepare empty dataframe to receive models' results
MLRresults<-as.data.frame(matrix(nrow=7, ncol=4))
colnames(MLRresults) <- (c("Model", "Deviance", "AIC", "AICc"))
for (i in 1:7){ # the remaining variables to be forward selected
  formula<-formlist[[i]]
  MLR_test<-multinom(formula, data=variables)
  MLRresults[i,1]<-paste(formlist[i]) # store the 'model formula'
  MLRresults[i,2]<-(Total_deviance-MLR_test[[19]])/Total_deviance # store the % of deviance explained
  MLRresults[i,3]<-MLR_test[[26]] # store the AIC value
  MLRresults[i,4]<-AICc(MLR_test)} # store the AICc value

wAICc<-Weights(MLRresults$AICc) # AIC weigth
MLRresults<-cbind(MLRresults, wAICc) # combine results in a single table
View(MLRresults[order(MLRresults$wAICc, decreasing=T), ]) # print the output table from the model selection procedure

##########################################################################################################################
# STEP 3 - Deviance partitioning among climatic stability, productivity, and topographic complexity

# Get the unique and shared contributions for the predictor sets related to contemporary environment
ind_fraction_table<-as.data.frame(matrix(nrow=8, ncol=2))
colnames(ind_fraction_table) <- (c("Individual fraction", "AdjRsq"))
ind_fraction_table[,1]<-c("[a]", # unique fraction = climatic stability
                          "[b]", # unique fraction = productivity
                          "[c]", # unique fraction = topography
                          "[d]", # share fraction = climatic stability and productivity
                          "[e]", # share fraction = climatic stability and topography
                          "[f]", # share fraction = productivity and topography
                          "[g]", # share fraction among the three predictor sets
                          "[h] = Residuals")
showvarparts(3)

# Store each individual fraction from 'a' to 'd'
ind_fraction_table[1,2]<-MLRresults[7,2] - MLRresults[5,2] # fraction 'a'
ind_fraction_table[2,2]<-MLRresults[7,2] - MLRresults[6,2] # fraction 'b'
ind_fraction_table[3,2]<-MLRresults[7,2] - MLRresults[4,2] # fraction 'c'
ind_fraction_table[4,2]<-MLRresults[6,2] - MLRresults[3,2] - ind_fraction_table[1,2] # fraction 'd'
ind_fraction_table[5,2]<-MLRresults[4,2] - MLRresults[2,2] - ind_fraction_table[2,2] # fraction 'e'
ind_fraction_table[6,2]<-MLRresults[4,2] - MLRresults[1,2] - ind_fraction_table[1,2] # fraction 'f'
ind_fraction_table[7,2]<-MLRresults[3,2] - ind_fraction_table[5,2] - ind_fraction_table[6,2] - ind_fraction_table[3,2] # fraction 'g'
ind_fraction_table[8,2]<- 1 - MLRresults[7,2] # fraction 'h'

##########################################################################################################################
# STEP 4 - Deviance partitioning among current climate, historical variation in climate, and topography

# Prepare the models to be tested
formlist <- list() # Empty list to receive nested formulas in forward selection
formlist[[1]]<-as.formula("subregion~AMT+APP+PPR+TAR") # current climate
formlist[[2]]<-as.formula("subregion~HDP+HDT+HHC+HTC") # historical variation in climate
formlist[[3]]<-as.formula("subregion~ElevR+ElevCV") # topographic complexity
formlist[[4]]<-as.formula("subregion~HDP+HDT+HHC+HTC+AMT+APP+TAR+PPR") # historical variation in climate + current climate
formlist[[5]]<-as.formula("subregion~HDP+HDT+HHC+HTC+ElevR+ElevCV") # historical variation in climate + topography
formlist[[6]]<-as.formula("subregion~AMT+APP+TAR+PPR+ElevR+ElevCV") # current climate + topography
formlist[[7]]<-as.formula("subregion~AMT+TAR+PPR+APP+ElevR+ElevCV+HDP+HDT+HHC+HTC") # Total 10 predictors

# Prepare empty dataframe to receive models' results
MLRresults<-as.data.frame(matrix(nrow=7, ncol=4))
colnames(MLRresults) <- (c("Model", "Deviance", "AIC", "AICc"))
for (i in 1:7){ # the remaining variables to be forward selected
  formula<-formlist[[i]]
  MLR_test<-multinom(formula, data=variables)
  MLRresults[i,1]<-paste(formlist[i]) # store the 'model formula'
  MLRresults[i,2]<-(Total_deviance-MLR_test[[19]])/Total_deviance # store the % of deviance explained
  MLRresults[i,3]<-MLR_test[[26]] # store the AIC value
  MLRresults[i,4]<-AICc(MLR_test)} # store the AICc value

wAICc<-Weights(MLRresults$AICc) # AIC weigth
MLRresults<-cbind(MLRresults, wAICc) # combine results in a single table
View(MLRresults[order(MLRresults$wAICc, decreasing=T), ]) # print the output table from the model selection procedure

# Get the unique and shared contributions for the predictor sets related to contemporary environment
ind_fraction_table<-as.data.frame(matrix(nrow=8, ncol=2))
colnames(ind_fraction_table) <- (c("Individual fraction", "AdjRsq"))
ind_fraction_table[,1]<-c("[a]", # unique fraction = current climate
                          "[b]", # unique fraction = historical variation in climate
                          "[c]", # unique fraction = topography
                          "[d]", # share fraction = current climate and historical variation in climate
                          "[e]", # share fraction = current climate and topography
                          "[f]", # share fraction = historical variation in climate and topography
                          "[g]", # share fraction among the three predictor sets
                          "[h] = Residuals")

# Store each individual fraction from 'a' to 'd'
ind_fraction_table[1,2]<-MLRresults[7,2] - MLRresults[5,2] # fraction 'a'
ind_fraction_table[2,2]<-MLRresults[7,2] - MLRresults[6,2] # fraction 'b'
ind_fraction_table[3,2]<-MLRresults[7,2] - MLRresults[4,2] # fraction 'c'
ind_fraction_table[4,2]<-MLRresults[6,2] - MLRresults[3,2] - ind_fraction_table[1,2] # fraction 'd'
ind_fraction_table[5,2]<-MLRresults[6,2] - MLRresults[1,2] - ind_fraction_table[3,2] # fraction 'e'
ind_fraction_table[6,2]<-MLRresults[5,2] - MLRresults[2,2] - ind_fraction_table[3,2] # fraction 'f'
ind_fraction_table[7,2]<-MLRresults[3,2] - ind_fraction_table[5,2] - ind_fraction_table[6,2] - ind_fraction_table[3,2] # fraction 'g'
ind_fraction_table[8,2]<- 1 - MLRresults[7,2] # fraction 'h'

##########################################################################################################################
# STEP 5 - Examine the spatial structure in the residuals of the best supported multinomial logistic regression model

# The best supported multinomial logistic regression (MLR) model
formula<-as.formula("subregion~HDP+HDT+HHC+HTC+AMT+APP+TAR+PPR")

# Extract the residuals from the MLR model
MLR_test<-multinom(formula, data=variables)
MLR_residuals<-residuals(MLR_test)

# Get the geographical coordinates of sites
coords<-rawdata[,1:2]

# Plot the Moran's I correlogram for the MLR model residuals
pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,1], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 1"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,2], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 2"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,3], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 3"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,4], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 4"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,5], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 5"), pch=16, lty=1, bty="n", cex=0.9)

pgi.cor<-as.data.frame(correlog(coords=coords, z=MLR_residuals[,6], method="Moran", nbclass=14))
plot(coef~dist.class, xlab="Distance class", ylab="Moran's I", pch=16, bty="l", cex.lab=1.2, ylim=c(-1,1), data=pgi.cor)
abline (h=0, lty=2, col="black")
lines(pgi.cor$dist.class, pgi.cor$coef, type = "l")
legend(12, 1, c("MLR residuals - Level 6"), pch=16, lty=1, bty="n", cex=0.9)